"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@yarnpkg/core");
const core_2 = require("@yarnpkg/core");
const core_3 = require("@yarnpkg/core");
const fslib_1 = require("@yarnpkg/fslib");
const querystring_1 = __importDefault(require("querystring"));
const constants_1 = require("./constants");
class ExecResolver {
    supportsDescriptor(descriptor, opts) {
        if (!descriptor.range.startsWith(constants_1.PROTOCOL))
            return false;
        return true;
    }
    supportsLocator(locator, opts) {
        if (!locator.reference.startsWith(constants_1.PROTOCOL))
            return false;
        return true;
    }
    shouldPersistResolution(locator, opts) {
        return false;
    }
    bindDescriptor(descriptor, fromLocator, opts) {
        if (descriptor.range.includes(`?`))
            return descriptor;
        return core_3.structUtils.makeDescriptor(descriptor, `${descriptor.range}?${querystring_1.default.stringify({
            locator: core_3.structUtils.stringifyLocator(fromLocator),
        })}`);
    }
    async getCandidates(descriptor, opts) {
        let path = descriptor.range;
        if (path.startsWith(constants_1.PROTOCOL))
            path = path.slice(constants_1.PROTOCOL.length);
        return [core_3.structUtils.makeLocator(descriptor, `${constants_1.PROTOCOL}${fslib_1.npath.toPortablePath(path)}`)];
    }
    async resolve(locator, opts) {
        const packageFetch = await opts.fetcher.fetch(locator, opts);
        const manifest = await core_3.miscUtils.releaseAfterUseAsync(async () => {
            return await core_1.Manifest.find(packageFetch.prefixPath, { baseFs: packageFetch.packageFs });
        }, packageFetch.releaseFs);
        return Object.assign(Object.assign({}, locator), { version: manifest.version || `0.0.0`, languageName: opts.project.configuration.get(`defaultLanguageName`), linkType: core_2.LinkType.HARD, dependencies: manifest.dependencies, peerDependencies: manifest.peerDependencies, dependenciesMeta: manifest.dependenciesMeta, peerDependenciesMeta: manifest.peerDependenciesMeta, bin: manifest.bin });
    }
}
exports.ExecResolver = ExecResolver;
